@@ -103,6 +103,12 @@ char*  kmail_chdir = NULL;
 #define DMODE_EXCLUDE 0
 #define DMODE_INCLUDE 1
 
+// Output type mode flags
+#define OTMODE_EMAIL        1
+#define OTMODE_APPOINTMENT  2
+#define OTMODE_JOURNAL      4
+#define OTMODE_CONTACT      8
+
 // output settings for RTF bodies
 // filename for the attachment
 #define RTF_ATTACH_NAME "rtf-body.rtf"
@@ -112,9 +118,11 @@ char*  kmail_chdir = NULL;
 // global settings
 int         mode         = MODE_NORMAL;
 int         mode_MH      = 0;   // a submode of MODE_SEPARATE
+int         mode_thunder = 0;   // a submode of MODE_RECURSE
 int         output_mode  = OUTPUT_NORMAL;
 int         contact_mode = CMODE_VCARD;
 int         deleted_mode = DMODE_EXCLUDE;
+int         output_type_mode = 0xff;    // Default to all.
 int         contact_mode_specified = 0;
 int         overwrite = 0;
 int         save_rtf_body = 1;
@@ -266,66 +274,90 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
             }
 
         } else if (item->contact && (item->type == PST_TYPE_CONTACT)) {
-            if (!ff.type) ff.type = item->type;
             DEBUG_INFO(("Processing Contact\n"));
-            if (ff.type != PST_TYPE_CONTACT) {
+            if (!(output_type_mode & OTMODE_CONTACT)) {
                 ff.skip_count++;
-                DEBUG_INFO(("I have a contact, but the folder type %"PRIi32" isn't a contacts folder. Skipping it\n", ff.type));
+                DEBUG_INFO(("skipping contact: not in output type list\n"));
             }
             else {
-                ff.item_count++;
-                if (mode == MODE_SEPARATE) mk_separate_file(&ff);
-                if (contact_mode == CMODE_VCARD) {
-                    pst_convert_utf8_null(item, &item->comment);
-                    write_vcard(ff.output, item, item->contact, item->comment.str);
+                if (!ff.type) ff.type = item->type;
+                if (ff.type != PST_TYPE_CONTACT) {
+                    ff.skip_count++;
+                    DEBUG_INFO(("I have a contact, but the folder type %"PRIi32" isn't a contacts folder. Skipping it\n", ff.type));
                 }
                 else {
-                    pst_convert_utf8(item, &item->contact->fullname);
-                    pst_convert_utf8(item, &item->contact->address1);
-                    fprintf(ff.output, "%s <%s>\n", item->contact->fullname.str, item->contact->address1.str);
+                    ff.item_count++;
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff);
+                    if (contact_mode == CMODE_VCARD) {
+                        pst_convert_utf8_null(item, &item->comment);
+                        write_vcard(ff.output, item, item->contact, item->comment.str);
+                    }
+                    else {
+                        pst_convert_utf8(item, &item->contact->fullname);
+                        pst_convert_utf8(item, &item->contact->address1);
+                        fprintf(ff.output, "%s <%s>\n", item->contact->fullname.str, item->contact->address1.str);
+                    }
                 }
             }
 
         } else if (item->email && ((item->type == PST_TYPE_NOTE) || (item->type == PST_TYPE_SCHEDULE) || (item->type == PST_TYPE_REPORT))) {
-            if (!ff.type) ff.type = item->type;
             DEBUG_INFO(("Processing Email\n"));
-            if ((ff.type != PST_TYPE_NOTE) && (ff.type != PST_TYPE_SCHEDULE) && (ff.type != PST_TYPE_REPORT)) {
+            if (!(output_type_mode & OTMODE_EMAIL)) {
                 ff.skip_count++;
-                DEBUG_INFO(("I have an email type %"PRIi32", but the folder type %"PRIi32" isn't an email folder. Skipping it\n", item->type, ff.type));
+                DEBUG_INFO(("skipping email: not in output type list\n"));
             }
             else {
-                char *extra_mime_headers = NULL;
-                ff.item_count++;
-                if (mode == MODE_SEPARATE) mk_separate_file(&ff);
-                write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                if (!ff.type) ff.type = item->type;
+                if ((ff.type != PST_TYPE_NOTE) && (ff.type != PST_TYPE_SCHEDULE) && (ff.type != PST_TYPE_REPORT)) {
+                    ff.skip_count++;
+                    DEBUG_INFO(("I have an email type %"PRIi32", but the folder type %"PRIi32" isn't an email folder. Skipping it\n", item->type, ff.type));
+                }
+                else {
+                    char *extra_mime_headers = NULL;
+                    ff.item_count++;
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff);
+                    write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                }
             }
 
         } else if (item->journal && (item->type == PST_TYPE_JOURNAL)) {
-            if (!ff.type) ff.type = item->type;
             DEBUG_INFO(("Processing Journal Entry\n"));
-            if (ff.type != PST_TYPE_JOURNAL) {
+            if (!(output_type_mode & OTMODE_JOURNAL)) {
                 ff.skip_count++;
-                DEBUG_INFO(("I have a journal entry, but the folder type %"PRIi32" isn't a journal folder. Skipping it\n", ff.type));
+                DEBUG_INFO(("skipping journal entry: not in output type list\n"));
             }
             else {
-                ff.item_count++;
-                if (mode == MODE_SEPARATE) mk_separate_file(&ff);
-                write_journal(ff.output, item);
-                fprintf(ff.output, "\n");
+                if (!ff.type) ff.type = item->type;
+                if (ff.type != PST_TYPE_JOURNAL) {
+                    ff.skip_count++;
+                    DEBUG_INFO(("I have a journal entry, but the folder type %"PRIi32" isn't a journal folder. Skipping it\n", ff.type));
+                }
+                else {
+                    ff.item_count++;
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff);
+                    write_journal(ff.output, item);
+                    fprintf(ff.output, "\n");
+                }
             }
 
         } else if (item->appointment && (item->type == PST_TYPE_APPOINTMENT)) {
-            if (!ff.type) ff.type = item->type;
             DEBUG_INFO(("Processing Appointment Entry\n"));
-            if (ff.type != PST_TYPE_APPOINTMENT) {
+            if (!(output_type_mode & OTMODE_APPOINTMENT)) {
                 ff.skip_count++;
-                DEBUG_INFO(("I have an appointment, but the folder type %"PRIi32" isn't an appointment folder. Skipping it\n", ff.type));
+                DEBUG_INFO(("skipping appointment: not in output type list\n"));
             }
             else {
-                ff.item_count++;
-                if (mode == MODE_SEPARATE) mk_separate_file(&ff);
-                write_appointment(ff.output, item, 0);
-                fprintf(ff.output, "\n");
+                if (!ff.type) ff.type = item->type;
+                if (ff.type != PST_TYPE_APPOINTMENT) {
+                    ff.skip_count++;
+                    DEBUG_INFO(("I have an appointment, but the folder type %"PRIi32" isn't an appointment folder. Skipping it\n", ff.type));
+                }
+                else {
+                    ff.item_count++;
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff);
+                    write_appointment(ff.output, item, 0);
+                    fprintf(ff.output, "\n");
+                }
             }
 
         } else if (item->message_store) {
@@ -364,7 +396,7 @@ int main(int argc, char* const* argv) {
     }
 
     // command-line option handling
-    while ((c = getopt(argc, argv, "bc:Dd:hj:kMo:qrSVw"))!= -1) {
+    while ((c = getopt(argc, argv, "bc:Dd:hj:kMo:qrSt:uVw"))!= -1) {
         switch (c) {
         case 'b':
             save_rtf_body = 0;
@@ -412,11 +444,46 @@ int main(int argc, char* const* argv) {
             break;
         case 'r':
             mode = MODE_RECURSE;
+            mode_thunder = 0;
             break;
         case 'S':
             mode = MODE_SEPARATE;
             mode_MH = 0;
             break;
+        case 't':
+            // email, appointment, contact, other
+            if (!optarg) {
+                usage();
+                exit(0);
+            }
+            temp = optarg;
+            output_type_mode = 0;
+            while (*temp > 0) {
+              switch (temp[0]) {
+                case 'e':
+                    output_type_mode |= OTMODE_EMAIL;
+                    break;
+                case 'a':
+                    output_type_mode |= OTMODE_APPOINTMENT;
+                    break;
+                case 'j':
+                    output_type_mode |= OTMODE_JOURNAL;
+                    break;
+                case 'c':
+                    output_type_mode |= OTMODE_CONTACT;
+                    break;
+                default:
+                    usage();
+                    exit(0);
+                    break;
+              }
+              temp++;
+            }
+            break;
+        case 'u':
+            mode = MODE_RECURSE;
+            mode_thunder = 1;
+            break;
         case 'V':
             version();
             exit(0);
@@ -586,6 +653,8 @@ void usage() {
     printf("\t-o <dirname>\t- Output directory to write files to. CWD is changed *after* opening pst file\n");
     printf("\t-q\t- Quiet. Only print error messages\n");
     printf("\t-r\t- Recursive. Output in a recursive format\n");
+    printf("\t-t[eajc]\t- Set the output type list. e = email, a = attachment, j = journal, c = contact\n");
+    printf("\t-u\t- Thunderbird mode. Write two extra .size and .type files\n");
     printf("\t-w\t- Overwrite any output mbox files\n");
     printf("\n");
     printf("Only one of -k -M -r -S should be specified\n");
@@ -905,8 +974,6 @@ void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *bound
 {
     pst_index_ll *ptr;
     DEBUG_ENT("write_embedded_message");
-    fprintf(f_output, "\n--%s\n", boundary);
-    fprintf(f_output, "Content-Type: %s\n\n", attach->mimetype.str);
     ptr = pst_getID(pf, attach->i_id);
 
     pst_desc_tree d_ptr;
@@ -922,8 +989,20 @@ void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *bound
     d_ptr.child_tail  = NULL;
 
     pst_item *item = pst_parse_item(pf, &d_ptr, attach->id2_head);
-    write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, 0, extra_mime_headers);
-    pst_freeItem(item);
+    // It appears that if the embedded message contains an appointment/
+    // calendar item, pst_parse_item returns NULL due to the presence of
+    // an unexpected reference type of 0x1048, which seems to represent
+    // an array of GUIDs representing a CLSID. It's likely that this is
+    // a reference to an internal Outlook COM class.
+    //      Log the skipped item and continue on.
+    if (!item) {
+        DEBUG_WARN(("write_embedded_message: pst_parse_item was unable to parse the embedded message in attachment ID %llu", attach->i_id));
+    } else {
+        fprintf(f_output, "\n--%s\n", boundary);
+        fprintf(f_output, "Content-Type: %s\n\n", attach->mimetype.str);
+        write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, 0, extra_mime_headers);
+        pst_freeItem(item);
+    }
 
     DEBUG_RET();
 }
@@ -1241,7 +1320,8 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
     // setup default body character set and report type
     strncpy(body_charset, pst_default_charset(item, sizeof(buffer_charset), buffer_charset), sizeof(body_charset));
     body_charset[sizeof(body_charset)-1] = '\0';
-    body_report[0] = '\0';
+    strncpy(body_report, "delivery-status", sizeof(body_report));
+    body_report[sizeof(body_report)-1] = '\0';
 
     // setup default sender
     pst_convert_utf8(item, &item->email->sender_address);
@@ -1344,11 +1424,12 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
 
     // print the supplied email headers
     if (headers) {
-        int len;
-        fprintf(f_output, "%s", headers);
-        // make sure the headers end with a \n
-        len = strlen(headers);
-        if (!len || (headers[len-1] != '\n')) fprintf(f_output, "\n");
+        int len = strlen(headers);
+        if (len > 0) {
+            fprintf(f_output, "%s", headers);
+            // make sure the headers end with a \n
+            if (headers[len-1] != '\n') fprintf(f_output, "\n");
+        }
     }
 
     // create required header fields that are not already written
@@ -1392,7 +1473,8 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
     // needed or used by mail clients
     pst_convert_utf8_null(item, &item->email->sender_address);
     if (item->email->sender_address.str && !strchr(item->email->sender_address.str, '@')
-                                        && strcmp(item->email->sender_address.str, ".")) {
+                                        && strcmp(item->email->sender_address.str, ".")
+                                        && (strlen(item->email->sender_address.str) > 0)) {
         fprintf(f_output, "X-libpst-forensic-sender: %s\n", item->email->sender_address.str);
     }
 
@@ -1403,7 +1485,7 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
 
     // add our own mime headers
     fprintf(f_output, "MIME-Version: 1.0\n");
-    if (body_report[0] != '\0') {
+    if (item->type == PST_TYPE_REPORT) {
         // multipart/report for DSN/MDN reports
         fprintf(f_output, "Content-Type: multipart/report; report-type=%s;\n\tboundary=\"%s\"\n", body_report, boundary);
     }
@@ -1413,7 +1495,7 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
     fprintf(f_output, "\n");    // end of headers, start of body
 
     // now dump the body parts
-    if ((item->email->report_text.str) && (body_report[0] != '\0')) {
+    if ((item->type == PST_TYPE_REPORT) && (item->email->report_text.str)) {
         write_body_part(f_output, &item->email->report_text, "text/plain", body_charset, boundary, pst);
         fprintf(f_output, "\n");
     }
@@ -1492,8 +1574,14 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
             pst_convert_utf8_null(item, &attach->filename2);
             pst_convert_utf8_null(item, &attach->mimetype);
             DEBUG_INFO(("Attempting Attachment encoding\n"));
-            if (!attach->data.data && attach->mimetype.str && !strcmp(attach->mimetype.str, RFC822)) {
-                DEBUG_INFO(("seem to have special embedded message attachment\n"));
+            if (attach->method == PST_ATTACH_EMBEDDED) {
+                DEBUG_INFO(("have an embedded rfc822 message attachment\n"));
+                if (attach->mimetype.str) {
+                    DEBUG_INFO(("which already has a mime-type of %s\n", attach->mimetype.str));
+                    free(attach->mimetype.str);
+                }
+                attach->mimetype.str = strdup(RFC822);
+                attach->mimetype.is_utf8 = 1;
                 find_rfc822_headers(extra_mime_headers);
                 write_embedded_message(f_output, attach, boundary, pst, extra_mime_headers);
             }
@@ -1814,9 +1902,14 @@ void create_enter_dir(struct file_ll* f, pst_item *item)
     DEBUG_ENT("create_enter_dir");
     if (mode == MODE_KMAIL)
         f->name = mk_kmail_dir(item->file_as.str);
-    else if (mode == MODE_RECURSE)
+    else if (mode == MODE_RECURSE) {
         f->name = mk_recurse_dir(item->file_as.str, f->type);
-    else if (mode == MODE_SEPARATE) {
+        if (mode_thunder) {
+            FILE *type_file = fopen(".type", "w");
+            fprintf(type_file, "%d\n", item->type);
+            fclose(type_file);
+        }
+    } else if (mode == MODE_SEPARATE) {
         // do similar stuff to recurse here.
         mk_separate_dir(item->file_as.str);
         f->name = (char*) pst_malloc(10);
@@ -1889,9 +1982,14 @@ void close_enter_dir(struct file_ll *f)
 
     if (mode == MODE_KMAIL)
         close_kmail_dir();
-    else if (mode == MODE_RECURSE)
+    else if (mode == MODE_RECURSE) {
+        if (mode_thunder) {
+            FILE *type_file = fopen(".size", "w");
+            fprintf(type_file, "%i %i\n", f->item_count, f->stored_count);
+            fclose(type_file);
+        }
         close_recurse_dir();
-    else if (mode == MODE_SEPARATE)
+    } else if (mode == MODE_SEPARATE)
         close_separate_dir();
 }
 
